import os
import shutil

import requests
import pytest

from .test_api import TEST_STACK_MODEL
from . import train_cron


@pytest.fixture
def api_token(auth_header):
    url = "http://rasa-x:5002/api/user"
    response = requests.get(url, headers=auth_header)
    return response.json().get("api_token")


def upload_test_model(auth_header, name):
    shutil.copyfile(f"{TEST_STACK_MODEL}.tar.gz", f"{name}.tar.gz")

    url = "http://rasa-x:5002/api/projects/default/models"
    with open(f"{name}.tar.gz", "rb") as f:
        response = requests.post(url, headers=auth_header, files={"model": f})
        response.raise_for_status()

    # tag model as production
    tag_url = (
        "http://rasa-x:5002/api/projects/default/models/"
        "{}/tags/production".format(name)
    )
    tag_response = requests.put(tag_url, headers=auth_header)
    tag_response.raise_for_status()


@pytest.fixture
def setup_environment(api_token):
    old_env = os.environ
    os.environ = os.environ.copy()
    os.environ["RASA_API_TOKEN"] = api_token
    os.environ["RASA_X_HOST"] = "http://rasa-x:5002"
    os.environ["TRAIN_CRON_TESTNG"] = True
    yield
    os.environ = old_env


def test_train_cron_with_upload(setup_environment, monkeypatch, auth_header):
    model_name = "test_cron_model_1"
    upload_test_model(auth_header, model_name)

    def always_better_score(a, b):
        return True

    monkeypatch.setattr(train_cron, "is_better_score", always_better_score)

    train_cron.main()
    production_model = train_cron.get_tagged_model(
        os.environ["RASA_X_HOST"], os.environ["RASA_API_TOKEN"], "production"
    )

    # Production model should have been replaced
    assert production_model != model_name


def test_train_cron_without_upload(setup_environment, monkeypatch, auth_header):
    model_name = "test_cron_model_2"
    upload_test_model(auth_header, model_name)

    def never_better_score(a, b):
        return False

    monkeypatch.setattr(train_cron, "is_better_score", never_better_score)

    train_cron.main()
    production_model = train_cron.get_tagged_model(
        os.environ["RASA_X_HOST"], os.environ["RASA_API_TOKEN"], "production"
    )

    # Production model should not have been replaced
    assert production_model == model_name
